@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2010-2011 Brazil
+/* Copyright(C) 2010-2013 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -12,13 +12,14 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "db.h"
 #include "pat.h"
 #include "ii.h"
 #include "util.h"
+#include "string_in.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -80,6 +81,148 @@ grn_inspect_name(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   return buf;
 }
 
+grn_obj *
+grn_inspect_encoding(grn_ctx *ctx, grn_obj *buf, grn_encoding encoding)
+{
+  switch (encoding) {
+  case GRN_ENC_DEFAULT :
+    GRN_TEXT_PUTS(ctx, buf, "default(");
+    grn_inspect_encoding(ctx, buf, grn_get_default_encoding());
+    GRN_TEXT_PUTS(ctx, buf, ")");
+    break;
+  case GRN_ENC_NONE :
+    GRN_TEXT_PUTS(ctx, buf, "none");
+    break;
+  case GRN_ENC_EUC_JP :
+    GRN_TEXT_PUTS(ctx, buf, "EUC-JP");
+    break;
+  case GRN_ENC_UTF8 :
+    GRN_TEXT_PUTS(ctx, buf, "UTF-8");
+    break;
+  case GRN_ENC_SJIS :
+    GRN_TEXT_PUTS(ctx, buf, "Shift_JIS");
+    break;
+  case GRN_ENC_LATIN1 :
+    GRN_TEXT_PUTS(ctx, buf, "Latin-1");
+    break;
+  case GRN_ENC_KOI8R :
+    GRN_TEXT_PUTS(ctx, buf, "KOI8-R");
+    break;
+  default :
+    GRN_TEXT_PUTS(ctx, buf, "unknown(");
+    grn_text_itoa(ctx, buf, encoding);
+    GRN_TEXT_PUTS(ctx, buf, ")");
+    break;
+  }
+
+  return buf;
+}
+
+grn_obj *
+grn_inspect_type(grn_ctx *ctx, grn_obj *buf, unsigned char type)
+{
+  switch (type) {
+  case GRN_VOID :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_VOID");
+    break;
+  case GRN_BULK :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_BULK");
+    break;
+  case GRN_PTR :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_PTR");
+    break;
+  case GRN_UVECTOR :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_UVECTOR");
+    break;
+  case GRN_PVECTOR :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_PVECTOR");
+    break;
+  case GRN_VECTOR :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_VECTOR");
+    break;
+  case GRN_MSG :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_MSG");
+    break;
+  case GRN_QUERY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_QUERY");
+    break;
+  case GRN_ACCESSOR :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_ACCESSOR");
+    break;
+  case GRN_SNIP :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_SNIP");
+    break;
+  case GRN_PATSNIP :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_PATSNIP");
+    break;
+  case GRN_STRING :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_STRING");
+    break;
+  case GRN_CURSOR_TABLE_HASH_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_CURSOR_TABLE_HASH_KEY");
+    break;
+  case GRN_CURSOR_TABLE_PAT_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_CURSOR_TABLE_PAT_KEY");
+    break;
+  case GRN_CURSOR_TABLE_DAT_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_CURSOR_TABLE_DAT_KEY");
+    break;
+  case GRN_CURSOR_TABLE_NO_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_CURSOR_TABLE_NO_KEY");
+    break;
+  case GRN_CURSOR_COLUMN_INDEX :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_CURSOR_COLUMN_INDEX");
+    break;
+  case GRN_CURSOR_COLUMN_GEO_INDEX :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_CURSOR_COLUMN_GEO_INDEX");
+    break;
+  case GRN_TYPE :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_TYPE");
+    break;
+  case GRN_PROC :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_PROC");
+    break;
+  case GRN_EXPR :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_EXPR");
+    break;
+  case GRN_TABLE_HASH_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_TABLE_HASH_KEY");
+    break;
+  case GRN_TABLE_PAT_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_TABLE_PAT_KEY");
+    break;
+  case GRN_TABLE_DAT_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_TABLE_DAT_KEY");
+    break;
+  case GRN_TABLE_NO_KEY :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_TABLE_NO_KEY");
+    break;
+  case GRN_DB :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_DB");
+    break;
+  case GRN_COLUMN_FIX_SIZE :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_COLUMN_FIX_SIZE");
+    break;
+  case GRN_COLUMN_VAR_SIZE :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_COLUMN_VAR_SIZE");
+    break;
+  case GRN_COLUMN_INDEX :
+    GRN_TEXT_PUTS(ctx, buf, "GRN_COLUMN_INDEX");
+    break;
+  default:
+    {
+      char type_in_hex[5]; /* "0xXX" */
+      sprintf(type_in_hex, "%#02x", type);
+      GRN_TEXT_PUTS(ctx, buf, "(unknown: ");
+      GRN_TEXT_PUTS(ctx, buf, type_in_hex);
+      GRN_TEXT_PUTS(ctx, buf, ")");
+    }
+    break;
+  }
+
+  return buf;
+}
+
 static grn_rc
 grn_proc_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 {
@@ -100,6 +243,9 @@ grn_proc_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   case GRN_PROC_HOOK :
     GRN_TEXT_PUTS(ctx, buf, "hook");
     break;
+  case GRN_PROC_NORMALIZER :
+    GRN_TEXT_PUTS(ctx, buf, "normalizer");
+    break;
   }
   GRN_TEXT_PUTS(ctx, buf, " ");
 
@@ -398,12 +544,23 @@ grn_table_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   } else {
     grn_table_cursor *tc;
     grn_obj *default_tokenizer;
+    grn_obj *normalizer;
 
     GRN_TEXT_PUTS(ctx, buf, " default_tokenizer:");
     default_tokenizer = grn_obj_get_info(ctx, obj,
                                          GRN_INFO_DEFAULT_TOKENIZER, NULL);
     if (default_tokenizer) {
       grn_inspect_name(ctx, buf, default_tokenizer);
+      grn_obj_unlink(ctx, default_tokenizer);
+    } else {
+      GRN_TEXT_PUTS(ctx, buf, "(nil)");
+    }
+
+    GRN_TEXT_PUTS(ctx, buf, " normalizer:");
+    normalizer = grn_obj_get_info(ctx, obj, GRN_INFO_NORMALIZER, NULL);
+    if (normalizer) {
+      grn_inspect_name(ctx, buf, normalizer);
+      grn_obj_unlink(ctx, normalizer);
     } else {
       GRN_TEXT_PUTS(ctx, buf, "(nil)");
     }
@@ -594,6 +751,9 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
   }
 
   switch (obj->header.type) {
+  case GRN_VOID :
+    /* TODO */
+    break;
   case GRN_BULK :
     switch (obj->header.domain) {
     case GRN_DB_TOKYO_GEO_POINT :
@@ -617,6 +777,9 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
       }
     }
     break;
+  case GRN_PTR :
+    /* TODO */
+    break;
   case GRN_UVECTOR :
     domain = grn_ctx_at(ctx, obj->header.domain);
     if (domain) {
@@ -633,19 +796,55 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
       }
     }
     break;
-  case GRN_EXPR :
-    grn_expr_inspect(ctx, buffer, obj);
+  case GRN_PVECTOR :
+    /* TODO */
+    break;
+  case GRN_VECTOR :
+    /* TODO */
+    break;
+  case GRN_MSG :
+    /* TODO */
+    break;
+  case GRN_ACCESSOR :
+    grn_accessor_inspect(ctx, buffer, obj);
+    return buffer;
+  case GRN_SNIP :
+  case GRN_PATSNIP :
+    /* TODO */
+    break;
+  case GRN_STRING :
+    grn_string_inspect(ctx, buffer, obj);
+    break;
+  case GRN_CURSOR_TABLE_HASH_KEY :
+    /* TODO */
+    break;
+  case GRN_CURSOR_TABLE_PAT_KEY :
+    grn_pat_cursor_inspect(ctx, (grn_pat_cursor *)obj, buffer);
+    return buffer;
+  case GRN_CURSOR_TABLE_DAT_KEY :
+  case GRN_CURSOR_TABLE_NO_KEY :
+  case GRN_CURSOR_COLUMN_INDEX :
+  case GRN_CURSOR_COLUMN_GEO_INDEX :
+    /* TODO */
+    break;
+  case GRN_TYPE :
+    grn_type_inspect(ctx, buffer, obj);
     return buffer;
   case GRN_PROC :
     grn_proc_inspect(ctx, buffer, obj);
     return buffer;
-  case GRN_ACCESSOR :
-  case GRN_ACCESSOR_VIEW :
-    grn_accessor_inspect(ctx, buffer, obj);
+  case GRN_EXPR :
+    grn_expr_inspect(ctx, buffer, obj);
     return buffer;
-  case GRN_TYPE :
-    grn_type_inspect(ctx, buffer, obj);
+  case GRN_TABLE_HASH_KEY :
+  case GRN_TABLE_PAT_KEY :
+  case GRN_TABLE_DAT_KEY :
+  case GRN_TABLE_NO_KEY :
+    grn_table_inspect(ctx, buffer, obj);
     return buffer;
+  case GRN_DB :
+    /* TODO */
+    break;
   case GRN_COLUMN_FIX_SIZE :
     grn_ra_inspect(ctx, buffer, obj);
     return buffer;
@@ -655,15 +854,6 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
   case GRN_COLUMN_INDEX :
     grn_ii_inspect(ctx, buffer, obj);
     return buffer;
-  case GRN_CURSOR_TABLE_PAT_KEY :
-    grn_pat_cursor_inspect(ctx, (grn_pat_cursor *)obj, buffer);
-    return buffer;
-  case GRN_TABLE_HASH_KEY :
-  case GRN_TABLE_PAT_KEY :
-  case GRN_TABLE_DAT_KEY :
-  case GRN_TABLE_NO_KEY :
-    grn_table_inspect(ctx, buffer, obj);
-    return buffer;
   default :
     break;
   }
@@ -720,7 +910,7 @@ grn_win32_base_dir(void)
         win32_base_dir = ".";
       } else {
         char *path;
-        win32_base_dir = malloc(ansi_dll_filename_size);
+        win32_base_dir = malloc(ansi_dll_filename_size + 1);
         WideCharToMultiByte(CP_ACP, 0,
                             absolute_dll_filename, absolute_dll_filename_size,
                             win32_base_dir, ansi_dll_filename_size,
